function [x,er,g]=TrustMin(fname,x,opt,varargin)
% (c) 2004 by Marco Maffezzoli. Ver. 1.1.2

if nargin==2|isempty(opt)
    tolx=sqrt(eps);
    tolg=eps^(1/3);
    maxeval=500;
    dbar=1e10;
    eta=0.05;
    alg=1;
else
    tolx=max(opt(1),eps);
    toly=max(opt(2),eps);
    maxeval=max(round(opt(3)),1);
    dbar=max(opt(4),10);
    eta=max(opt(5),1e-4);
    alg=max(round(opt(6)),1);
end
er=0;
[xr,xc]=size(x);
y=feval(fname,x,varargin{:});
[g,H]=gradhess(fname,x,y,varargin{:});
ng0=norm(g);
if ng0<=tolg
    return
end
ng=ng0;
cp=ng^3/(g*H*g');
if ~isfinite(cp)
    error('The initial Cauchy step is NaN or Inf: rescale the problem!')
end
delta=max(1,min(abs(cp),dbar));
[s,ns]=steihaug(g',H,ng,delta);
for k=1:maxeval
    tol=tolx*(1+norm(x));
    if ns<=tol
        if ng>tolg*(1+ng0)
            er=1;
        end
        return
    end
    x1=reshape(x(:)+s,xr,xc);
    y1=feval(fname,x1,varargin{:});
    rho=(y1-y)/(g*s+s'*H*s/2);
    if rho<0.25
        delta=delta/4;
        if delta<=tol
            if ng>tolg*(1+ng0)
                er=1;
            end
            return
        end        
    elseif (rho>0.75)&(ns>=delta)
        delta=min(2*delta,dbar);
    end
    if rho>eta
        x=x1;
        y=y1;
        [g,H]=gradhess(fname,x,y,varargin{:});
        ng=norm(g);
        if ng<=tolg
            return
        end
    end
    [s,ns]=steihaug(g',H,ng,delta);
end
er=2;