% MainFile basic_example2.m
% estimation of fiscal and monetary policy rules

%%%%%%%%%%%%%%%%%%%%%% Exercises %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% exercise 1: Change the number of instruments in equation 1 and then in equation 2.
% exercise 2: Change the sample. 
% exercise 3: Compare just -identified and overidentified estimates.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global nz1 nz2

%=======================================================================
% LOAD DATA
%=======================================================================
%data = xlsread('BCEP_us.xls');
data = xlsread('data_example2_US');

%=======================================================================
% SAMPLE 
%=======================================================================
sample=[1:151];
%sample=[63:151];

%=======================================================================
% ESTIMATION
%=======================================================================
dates=data(sample,1);
scdebt=1;

% Instruments
deficit_2=data(sample,16);
gap_2=data(sample,17);
gap_s_2=data(sample,18);
gap_b_2=data(sample,19);
debt_2=data(sample,20)./scdebt;
cpi_2=data(sample,21);
r_2=data(sample,22);

deficit_3=data(sample,23);
gap_3=data(sample,24);
gap_s_3=data(sample,25);
gap_b_3=data(sample,26);
debt_3=data(sample,27)./scdebt;
cpi_3=data(sample,28);
r_3=data(sample,29);

% lhs variables
deficit=data(sample,2);
Y1=deficit;

r=data(sample,8);
Y2=r;

% rhs variables, system F and system M
deficit_1=data(sample,9);
gap_s=data(sample,4);
gap_b=data(sample,5);
debt=data(sample,6)./scdebt;
X1=[deficit_1 gap_s gap_b debt]; %FISCAL POLICY

gap=data(sample,3);
cpi=data(sample,7);
r_1=data(sample,15);
%X2=[cpi gap ];        %MONETARY POLICY 
X2=[cpi gap r_1];      %MONETARY POLICY including interest rate

% instruments, system F and system M
gap_s_1=data(sample,11);
gap_b_1=data(sample,12);
debt_1=data(sample,13)./scdebt;
cpi_1=data(sample,14);
gap_1=data(sample,10);

% Z1 are instruments for the first equation, z2 are instruments for the
% second equation
Z1=[deficit_1 gap_s_1 gap_b_1 debt_1 cpi_1];

%Z2=[cpi_1 gap_1 r_1];    %in case of whole sample but no overidentific
Z2=[cpi_1 gap_1 r_1 cpi_2];         %in case of reduced sample (passive)
%Z2=[cpi_1 gap_1 r_1 cpi_3];         %in case of reduced sample (active)

ns=2;
nx1=4;
nx2=2;
nobs=length(sample);
maxiter=200;
stprl=sqrt(eps);
nz1=5;
%[Y1,X1,Z1]=genera_IV_model(nobs,nz1-1);
nz2=2;
%[Y2,X2,Z2]=genera_IV_model(nobs,nz2-1);
X1=[ones(nobs,1) X1]; % adding constant in the fiscal policy
%X2=[ones(nobs,1) X2];  % adding constant in the monetary policy
c1=size(X1,2);
Z1=[ones(nobs,1) Z1];
%Z2=[ones(nobs,1) Z2];

npar=size(X1,2)+size(X2,2);

%W=[ 0.0140    0.0257    0.0051    0.0085    0.2284   -0.1104    0.0036;...
%    0.0257    0.0474    0.0093    0.0157    0.4200   -0.2029    0.0065;...
%    0.0051    0.0093    0.0021    0.0031    0.0827   -0.0400    0.0011;...
%    0.0085    0.0157    0.0031    0.0053    0.1382   -0.0669    0.0022;...
%    0.2284    0.4200    0.0827    0.1382    3.7324   -1.8024    0.0583;...
%   -0.1104   -0.2029   -0.0400   -0.0669   -1.8024    0.8706   -0.0281;...
%    0.0036    0.0065    0.0011    0.0022    0.0583   -0.0281    0.0010];
%W=(W+eye(nz1+nz2))*100;

b0=zeros(1,npar);
opt=[];
nW=0;
delta=1;
nz1=size(Z1,2);
nz2=size(Z2,2);
W=eye(nz1+nz2);
y0=0;
y1=0;

%while and((delta>stprl*max([Y1 Y2],[],1)), (nW<=maxiter))
while and(delta>stprl./100,(nW<=maxiter))
      nW=nW+1;
      [b,er,G]=TrustMin('gmmopt',b0,opt,Y1,Y2,X1,X2,Z1,Z2,W,nobs,nz1,nz2);
      delta=((b-b0)*(b-b0)').^0.5;
      b0=b;
      D1=-Z1'*X1./nobs;
      D2=-Z2'*X2./nobs;
      D=[D1 zeros(nz1,npar-c1); zeros(nz2,c1) D2];
      m1=gmmmid(b0(1:c1),Y1,X1,Z1,nz1);
      m2=gmmmid(b0(c1+1:npar),Y2,X2,Z2,nz2);
      m=[m1 m2];
      
      % plotting the instruments
      %figure(001);
      %subplot(3,2,1), plot(m1(:,1)); legend('deficit_1',4);
      %subplot(3,2,2), plot(m1(:,2)); legend('gap_s_1',4);
      %subplot(3,2,3), plot(m1(:,3)); legend('gap_b_1',4);
      %subplot(3,2,4), plot(m1(:,4)); legend('debt_1',4);
      %subplot(3,2,5), plot(m1(:,5)); legend('cpi_1',4);
      %subplot(3,2,6), plot(m1(:,6));
      
      %line(sample,m1);
      %figure(002);
      %subplot(2,2,1), plot(m2(:,1)); legend('cpi_1',4);
      %subplot(2,2,2), plot(m2(:,2)); legend('gap_1',4);
      %subplot(2,2,3), plot(m2(:,3)); legend('r_1',4);
      %subplot(2,2,4), plot(m2(:,4));
      
      m1=gmmmom(b0(1:c1),Y1,X1,Z1,nobs,nz1);
      m2=gmmmom(b0(c1+1:npar),Y2,X2,Z2,nobs,nz2);
      mm=[m1;m2];
      Jtest=mm'*W*mm;
      S=vcovHAC(m,1);
      W=S\eye(nz1+nz2);
      Wb=(D'*W*D)\eye(npar);
      
      %equality test for gap_s and gap_b coefficients 
      pool=(b(3)-b(4))*(ones(1,2)*Wb(3:4,3:4)*ones(2,1))*(b(3)-b(4));
end
serb=diag(Wb).^0.5;
ttest=b'./serb;
%[b'-1.96*serb b' b'+1.96*serb]

disp('*****************')
disp('     Results')
disp(' Estimate, standard error, t-test')
disp('*****************')
[b' serb ttest]

disp(' J-statistics')
nobs*Jtest

disp(' Statistics of equality of coefficients on output')
nobs*pool